Option Explicit

Dim dicHelpLong, dicHelpShort
Dim blnAdditional, blnDebug, blnOverwrite, blnWindowsOnly
Dim intBitsOS, intValidArgs, i, j
Dim colItems, objExec, objHTMLFile, objFolder
Dim objFolderItem, objFSO, objItem, objKey, objMatches, objMatches2
Dim objRE, objShell, objWMISvc, wshShell
Dim strAlphabet, strCommand, strCSDVer, strFile, strFirstLetter, strHelpAll
Dim strHelpLong, strHelpShort, strHead, strHTML, strMsg, strNumVer, strOSLocl
Dim strPattern, strPreviousLetter, strScriptVer, strUnknownCommand, strWinVer

Const MY_DOCUMENTS = &H5

Const dictKey  = 1
Const dictItem = 2

Const TristateFalse      =  0
Const TristateMixed      = -2
Const TristateTrue       = -1
Const TristateUseDefault = -2

Const ForAppending = 8
Const ForReading   = 1
Const ForWriting   = 2

strMsg         = ""
strScriptVer   = "2.12"
intValidArgs   = 0
blnAdditional  = False
blnDebug       = False
blnOverwrite   = False
blnWindowsOnly = False

Set objFSO = CreateObject( "Scripting.FileSystemObject" )

With WScript.Arguments
	If .Named.Exists( "DEBUG" ) Then
		blnDebug     = True
		intValidArgs = intValidArgs + 1
		WScript.Echo DebugDisplay( "Show intermediate results" )
	End If
	If .Named.Exists( "WHO" ) Then
		blnWindowsOnly = True
		intValidArgs   = intValidArgs + 1
		If blnDebug Then WScript.Echo DebugDisplay( "Show ""all"" commands" )
	Else
		If blnDebug Then WScript.Echo DebugDisplay( "Show commands listed by HELP only" )
	End If
	If .Named.Exists( "Y" ) Then
		blnOverwrite = True
		intValidArgs = intValidArgs + 1
		If blnDebug Then WScript.Echo DebugDisplay( "File overwrite ON" )
	Else
		If blnDebug Then WScript.Echo DebugDisplay( "File overwrite OFF" )
	End If
	If .Unnamed.Count > 0 Then
		strFile = .Unnamed(0)
		intValidArgs = intValidArgs + 1
		If blnDebug Then WScript.Echo DebugDisplay( "Output file: " & strFile )
	Else
		' Find the path to "My Documents"
		Set objShell      = CreateObject( "Shell.Application" )
		Set objFolder     = objShell.Namespace( MY_DOCUMENTS )
		Set objFolderItem = objFolder.Self
		strFile = objFSO.BuildPath( objFolderItem.Path, "allhelp.html" )
		If blnDebug Then WScript.Echo DebugDisplay( "Output file: " & strFile )
		Set objFolderItem = Nothing
		Set objFolder     = Nothing
		Set objShell      = Nothing
	End If
	If intValidArgs <> .Count Then
		strMsg = strMsg & "ERROR: Invalid command line arguments" & vbCrLf & vbCrLf
		Syntax
	End If
	If objFSO.FileExists( strFile ) And Not blnOverwrite Then
		strMsg = strMsg & "ERROR: The specified output file exists and file overwrite is OFF" & vbCrLf & vbCrLf
		Syntax
	End If
	If objFSO.FolderExists( strFile ) Then
		strMsg = strMsg & "ERROR: A folder with the name of the specified output file exists" & vbCrLf & vbCrLf
		Syntax
	End If
	If Not objFSO.FolderExists( objFSO.GetParentFolderName( strFile ) ) Then
		strMsg = strMsg & "ERROR: The parent folder for the specified output file does not exists" & vbCrLf & vbCrLf
		Syntax
	End If
End With

' Get the OS version
Set objWMISvc = GetObject( "winmgmts://./root/cimv2" )
Set colItems  = objWMISvc.ExecQuery( "SELECT * FROM Win32_OperatingSystem", , 48 )
For Each objItem in colItems
	strWinVer = objItem.Caption
	strCSDVer = objItem.CSDVersion
	strOSLocl = objItem.OSLanguage
	strNumVer = objItem.Version
Next
Set colItems  = objWMISvc.ExecQuery( "SELECT * FROM Win32_Processor", , 48 )
For Each objItem in colItems
	intBitsOS = objItem.AddressWidth
Next
Set colItems  = Nothing
Set objWMISvc = Nothing

' Instantiate wshShell, as we'll use it regularly
Set wshShell = CreateObject( "WScript.Shell" )

' Retrieve the error message for unknown commands
strUnknownCommand = GetCommandError( )

' Create a Dictionary objects to contain the commands and associated help texts
Set dicHelpLong  = CreateObject( "Scripting.Dictionary" )
Set dicHelpShort = CreateObject( "Scripting.Dictionary" )

' Fill the commands lists
Set objExec  = wshShell.Exec( "CMD.EXE /A /C (TITLE AllHelp: HELP&HELP)" )
strHelpAll = objExec.StdOut.ReadAll
objExec.Terminate
Set objExec = Nothing
Set objRE   = New RegExp
objRE.Global     = True
objRE.IgnoreCase = False
objRE.Pattern    = "(^|[\n\r])[A-Z]+ +([^\n\r]+)(\r\n +([^\n\r]+))*"
Set objMatches = objRE.Execute( strHelpAll )
For i = 0 To objMatches.Count - 1
	strCommand   = Mid( objMatches.Item(i), 2, InStr( objMatches.Item(i), " " ) - 2 )
	strHelpShort = SuperTrim( Mid( objMatches.Item(i), InStr( objMatches.Item(i), " " ) ) )
	dicHelpShort.Add strCommand, strHelpShort
	dicHelpLong.Add  strCommand, GetCmdHelp( strCommand )
Next
Set objMatches = Nothing
Set objRE      = Nothing

' Display intermediate results for debugging purposes
If blnDebug Then WScript.Echo DebugDisplayHelpFound( )

If Not blnWindowsOnly Then
	AddCommand "APPVERIF",    "[^\r\n]+",                                     "[^\r\n]+"
	AddCommand "ARP",         "\r\n[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",     "(\r\n[^\r\n]+)+"
	AddCommand "AT",          "([^ ][^\r\n]+\r\n)+",                          "([^\r\n]+\r\n)+"
	AddCommand "BITSADMIN",   "\r\n[^\r\n\(\[]+\r\n([^\r\n\(\[]+\r\n)*",      "([^\r\n]+\r\n)+"
	AddCommand "BOOTCFG",     ":\r\n +[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",  "(\r\n +[^\r\n]+)+"
	AddCommand "CHOICE",      "[\r\n]+CHOICE +\[ */[^\r\n]+[\r\n]+[A-Za-z ]+:[\r\n]+( +[^\/\r\n]+[\r\n]+)+[A-Za-z ]+:[\r\n]", "([\r\n]+ +[^\/\r\n]+)+[\r\n]"
	AddCommand "CIPHER",      "([^ ][^\r\n]+\r\n)+",                          "([^\r\n]+\r\n)+"
	AddCommand "CLIP",        ":\r\n +[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",  "([^:][^\r\n]+\r\n)+"
	AddCommand "CMDKEY",      "\r\n[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",     "(\r\n[^\r\n]+)+"
	AddCommand "CSVDE",       "\r\n[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",     "(\r\n[^\r\n]+)+"
	AddCommand "DCDIAG",      "([^\r\n]+[\r\n]{1,2})+",                       "([^\r\n]+[\r\n]{1,2})+"
	AddCommand "DEVCON",      "[A-Z][^\r\n\[]+:[\r\n]{2,3}",                  "[^:\r\n]+"
	AddCommand "DIANTZ",      "([^\r\n\[]+ [^\r\n\[]+[\r\n]+)+",              "(.|\r|\n)+"
	AddCommand "DISM",        ":\r?\n\r?\n( +[^\r\n]+\r?\n)+\r?\n",           "([^:][^\r\n]+\r?\n)+"
	AddCommand "DISKRAID",    "[\r\n]([^\r\n\(]+[\r\n])+",                    "[\r\n]([^\r\n\(]+[\r\n])+"
	AddCommand "DSACLS",      "[A-Z][^\r\n\[]+[\r\n]{2,3}",                   "[^\r\n]+"
	AddCommand "DSADD",       ": +[A-Z][^\r\n]+(\r\n[^\r\n]+)*:\r\n\r\n",     "[^:](.|\r|\n)+\."
	AddCommand "DSDBUTIL",    "\n([^ \r\n\(][^\r\n\(]+\r?\n)+\r?\n",          "(.|\r|\n)+"
	AddCommand "DSGET",       ": +[A-Z][^\r\n]+(\r\n[^\r\n]+)*:\r\n\r\n",     "[^:](.|\r|\n)+\."
	AddCommand "DSMOD",       ": +[A-Z][^\r\n]+(\r\n[^\r\n]+)*:\r\n\r\n",     "[^:](.|\r|\n)+\."
	AddCommand "DSMOVE",      ": +[A-Z][^\r\n]+(\r\n[^\r\n]+)*\r\n\r\n",      "[^:](.|\r|\n)+\."
	AddCommand "DSQUERY",     ": +[A-Z][^\r\n]+(\r\n[^\r\n]+)*:\r\n\r\n",     "[^:](.|\r|\n)+\."
	AddCommand "DSRM",        ": +[A-Z][^\r\n]+(\r\n[^\r\n]+)*\r\n\r\n",      "[^:](.|\r|\n)+\."
	AddCommand "EVENTCREATE", ":\r\n +[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",  "(\r\n +[^\r\n]+)+"
	AddCommand "EXPAND",      "\r\n\r\n[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n", "(\r\n[^\r\n]+)+"
	AddCommand "FINGER",      "\r\n[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",     "(\r\n[^\r\n]+)+"
	AddCommand "FORFILES",    ":\r\n +[A-Z][^\r\n]+(\r\n[^\r\n]+)*\r\n\r\n",  "(\r\n[^\r\n]+)+"
	AddCommand "FTP",         "\r\n[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",     "(\r\n[^\r\n]+)+"
	AddCommand "GETMAC",      ":\r\n +[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",  "(\r\n +[^\r\n]+)+"
	AddCommand "GPFIXUP",     "([^\r\n]+[\r\n]{2,3})+[\r\n]{2,3}",            "([^\r\n]+[\r\n]{2,3})+"
	AddCommand "GPUPDATE",    ": +[A-Z][^\r\n]+[\r\n]{2,3}([^\r\n]+[\r\n]{2,3})*[\r\n]{4,6}", "([^:][^\r\n]+[\r\n]{2,3})+"
	AddCommand "HOSTNAME",    "\r\n[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",     "(\r\n[^\r\n]+)+"
	AddCommand "IFMEMBER",    "\r\n[^/]+\r\n",                                "(.|\r|\n)+"
	AddCommand "IPCONFIG",    "\r\n\r\n[A-Z][^\r\n]+\r\n([^\r\n\s][^\r\n]+\r\n)+\r\n", "([^\r\n]+\r\n)+"
	AddCommand "ISCSICLI",    "([^\r\n]+ [^\r\n]+[\r\n]{1,2})+",              "(.|\r|\n)+"
	AddCommand "JT",          "[^\r\n]+",                                     "(.|\r|\n)+"
	AddCommand "LDIFDE",      "[\r\n]+([^\r\n]+ [^\r\n]+[\r\n]{1,2}[^=])+",   "([^\r\n]+ [^\r\n]+[\r\n]+)+"
	AddCommand "LODCTR",      "\r\n +[A-Z][^\r\n]+\r\n( +[^\r\n]+\r\n)*\r\n", "([^\r\n]+\r\n)+"
	AddCommand "LOGEVENT",    ": +[^\r\n]+\r\n([^\r\n]+\r\n)*",               "[^:][^\r\n]+\r\n([^\r\n]+\r\n)*"
	AddCommand "LOGMAN",      "[\r\n]+[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",  "([^\r\n]+\r\n)+"
	AddCommand "LOGOFF",      "[^\r\n]+\r\n",                                 "[^\r\n]+"
	AddCommand "MAKECAB",     "([^\r\n\[]+ [^\r\n\[]+[\r\n]+)+",              "(.|\r|\n)+"
	AddCommand "MOUNTVOL",    "[\r\n]*[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",  "([^\r\n]+\r\n)+"
	AddCommand "NETSTAT",     "[\r\n]*[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",  "([^\r\n]+\r\n)+"
	AddCommand "NTDSUTIL",    "\n([^ \r\n\(][^\r\n\(]+\r?\n)+\r?\n",          "(.|\r|\n)+"
	AddCommand "PORTQRY",     "[\r\n]*[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",  "([^\r\n]+\r\n)+"
	AddCommand "POWERCFG",    ":\r\n +[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",  "(\r\n +[^\r\n]+)+"
	AddCommand "PSEXEC",      "- ([^\r\n\-]+[\r\n]{1,2})+",                   "[^-]([^\r\n]+\r\n)+"
	AddCommand "PSFILE",      "([\r\n]{1,2}[A-Za-z .]+ [A-Za-z .]+)+\.",      "(.|\r|\n)+"
	AddCommand "PSGETSID",    "- ([^\r\n\-]+[\r\n]{1,2})+",                   "[^-]([^\r\n]+\r\n)+"
	AddCommand "PSINFO",      "- ([^\r\n\-]+[\r\n]{1,2})+",                   "[^-]([^\r\n]+\r\n)+"
	AddCommand "PSKILL",      "- ([^\r\n\-]+[\r\n]{1,2})+",                   "[^-]([^\r\n]+\r\n)+"
	AddCommand "PSLIST",      "- ([^\r\n\-]+[\r\n]{1,2})+",                   "[^-]([^\r\n]+\r\n)+"
	AddCommand "PSLOGGEDON",  "- ([^\r\n\-]+[\r\n]{1,2})+",                   "[^-]([^\r\n]+\r\n)+"
	AddCommand "PSLOGLIST",   "- ([^\r\n\-]+[\r\n]{1,2})+",                   "[^-]([^\r\n]+\r\n)+"
	AddCommand "PSPASSWD",    "- ([^\r\n\-]+[\r\n]{1,2})+",                   "[^-]([^\r\n]+\r\n)+"
	AddCommand "PSSERVICE",   "- ([^\r\n\-]+[\r\n]{1,2})+",                   "[^-]([^\r\n]+\r\n)+"
	AddCommand "PSSHUTDOWN",  "- ([^\r\n\-]+[\r\n]{1,2})+",                   "[^-]([^\r\n]+\r\n)+"
	AddCommand "PSSUSPEND",   "- ([^\r\n\-]+[\r\n]{1,2})+",                   "[^-]([^\r\n]+\r\n)+"
	AddCommand "REAGENTC",    "([^\r\n\[]+[\r\n]+)+",                         "(.|\r|\n)+"
	AddCommand "RECOVER",     "[\r\n]*[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",  "([^\r\n]+\r\n)+"
	AddCommand "REGISTER-CIMPROVIDER", "([^\r\n\:]+[\r\n]+)+",                "(.|\r|\n)+"
	AddCommand "RELOG",       "[\r\n]+[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)+\r\n",  "([^\r\n]+\r\n)+"
	AddCommand "RENDOM",      "[\r\n]([^\r\n\(\:]+[\r\n]+)+",                 "(.|\r|\n)+"
	AddCommand "ROUTE",       "\r\n[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)*\r\n",     "([^\r\n]+\r\n)+"
	AddCommand "SFC",         "[\r\n]{2,3}[A-Z][^\r\n\(]+[\r\n]{2,3}([^\r\n]+[\r\n]{2,3})*[\r\n]{2,3}", "([^\r\n]+[\r\n]{2,3})+"
	AddCommand "SETX",        ":\r\n +[^\r\n/]+\r\n( +[^\r\n/]+\r\n)*",       "[^:](.|\r|\n)+"
	AddCommand "SOON",        ":[^:]*:",                                      "[^:]+"
	AddCommand "SUBINACL",    "([^\r\n]+\r\n)+",                              "([^\r\n]+\r\n)+"
	AddCommand "TAKEOWN",     ":\r\n +[^\r\n]+\r\n( +[^\r\n]+\r\n)*",         "([^:\r\n][^\r\n]+\r\n)+"
	AddCommand "TIMEOUT",     ":\r\n +[^\r\n]+\r\n( +[^\r\n]+\r\n)*",         "[^:](.|\r|\n)+"
	AddCommand "TYPEPERF",    "[\r\n]+[A-Z][^\r\n]+\r\n([^\r\n]+\r\n)+\r\n",  "([^\r\n]+\r\n)+"
	AddCommand "UNLODCTR",    "\r\n +[A-Z][^\r\n]+\r\n( +[^\r\n]+\r\n)*\r\n", "([^\r\n]+\r\n)+"
	AddCommand "VSSADMIN",    "[^\(\r\n]+\r\n([^\(\r\n\(]+)*",                "([^\r\n]+\r\n)+"
	AddCommand "WAITFOR",     ":\r\n +[^\r\n]+\r\n( +[^\r\n]+\r\n)*",         "[^:](.|\r|\n)+"
	AddCommand "WHERE",       ":\r\n +[^\r\n]+\r\n( +[^\r\n]+\r\n)*",         "[^:](.|\r|\n)+"
	AddCommand "WHOAMI",      "[^\d]:\r\n +[^\r\n]+\r\n( +[^\r\n]+\r\n)*",    "[^:]{2}(.|\r|\n)+"
	AddCommand "WINRM",       "\r\n[^\r\n]+\r\n([^\r\n]+\r\n)*",              "([^\r\n]+\r\n)+"

	AddCommandWithoutSummary "AUDITPOL"
	AddCommandWithoutSummary "CSCRIPT"
	AddCommandWithoutSummary "DISKPERF"
	AddCommandWithoutSummary "FLTMC"
	AddCommandWithoutSummary "IISRESET"
	AddCommandWithoutSummary "MRINFO"
	AddCommandWithoutSummary "NET"
	AddCommandWithoutSummary "NETDOM"
	AddCommandWithoutSummary "NETSH"
	AddCommandWithoutSummary "NSLOOKUP"
	AddCommandWithoutSummary "PATHPING"
	AddCommandWithoutSummary "PING"
	AddCommandWithoutSummary "RASDIAL"
	AddCommandWithoutSummary "REDIRCMP"
	AddCommandWithoutSummary "REDIRUSR"
	AddCommandWithoutSummary "REG"
	AddCommandWithoutSummary "REGINI"
	AddCommandWithoutSummary "REPADMIN"
	AddCommandWithoutSummary "RPCPING"
	AddCommandWithoutSummary "SDBINST"
	AddCommandWithoutSummary "SECEDIT"
	AddCommandWithoutSummary "SHORTCUT"
	AddCommandWithoutSummary "SXSTRACE"
	AddCommandWithoutSummary "TRACERPT"
	AddCommandWithoutSummary "TRACERT"
	AddCommandWithoutSummary "TZUTIL"
	AddCommandWithoutSummary "VERIFIER"
	AddCommandWithoutSummary "W32TM"
	AddCommandWithoutSummary "WECUTIL"
	AddCommandWithoutSummary "WEVTUTIL"
	AddCommandWithoutSummary "WINRS"
	AddCommandWithoutSummary "WMIC"
	
	' Extra check for RUNAS, because it doesn't always use Standard Output
	If GetAddedCmdHelp( "RUNAS" ) <> "" Then
		AddCommandWithoutSummary "RUNAS"
	End If
	
	' Sort the list of commands
	SortDictionary dicHelpShort, dictKey
	
	' Display intermediate results for debugging purposes
	If blnDebug Then WScript.Echo DebugDisplayHelpFound( )
End If

' Create the head for the HTML page
strHead = "<!DOCTYPE HTML PUBLIC ""-//W3C//DTD HTML 4.01//EN"" ""http://www.w3.org/TR/html4/strict.dtd"">" _
        & vbCrLf _
        & "<html>" _
        & vbCrLf _
        & "<head>" _
        & vbCrLf & vbCrLf _
        & "<title>Help for all " & strWinVer & " " & strCSDVer & " commands</title>" _
        & vbCrLf & vbCrLf _
        & "<meta name=""generator"" content=""AllHelp.vbs, Version " & strScriptVer & ", by Rob van der Woude, www.robvanderwoude.com"">" _
        & vbCrLf _
        & "<style type=""text/css"">" _
        & vbCrLf _
        & "td.Command" _
        & vbCrLf _
        & "{" _
        & vbCrLf _
        & vbTab & "vertical-align: top;" _
        & vbCrLf _
        & vbTab & "padding-left: 0px;" _
        & vbCrLf _
        & vbTab & "padding-right: 5px;" _
        & vbCrLf _
        & vbTab & "font-weight: bold;" _
        & vbCrLf _
        & vbTab & "white-space: nowrap;" _
        & vbCrLf _
        & "}" _
        & vbCrLf & vbCrLf _
        & "th" _
        & vbCrLf _
        & "{" _
        & vbCrLf _
        & vbTab & "vertical-align: top;" _
        & vbCrLf _
        & "}" _
        & vbCrLf & vbCrLf _
        & "table.Alphabet th" _
        & vbCrLf _
        & "{" _
        & vbCrLf _
        & vbTab & "padding: 10px;" _
        & vbCrLf _
        & "}" _
        & vbCrLf & vbCrLf _
        & "pre" _
        & vbCrLf _
        & "{" _
        & vbCrLf _
        & vbTab & "white-space: pre-wrap;" _
        & vbCrLf _
        & "}" _
        & vbCrLf & vbCrLf _
        & "</style>" _
        & vbCrLf & vbCrLf _
        & "</head>" _
        & vbCrLf & vbCrLf _
        & "<body>" _
        & vbCrLf & vbCrLf _
        & "<div style=""text-align: center;"">" _
        & vbCrLf _
        & "<h1>" & strWinVer & " " & Trim( strCSDVer ) & "</h1>" _
        & vbCrLf _
        & "<h2>Version " & strNumVer & " (" & intBitsOS & " bits)</h2>" _
        & vbCrLf _
        & "<h3>" & GetOSLanguage( strOSLocl ) & "</h3>" _
        & vbCrLf _
        & "</div>" _
        & vbCrLf & vbCrLf _
        & "<p>&nbsp;</p>" _
        & vbCrLf & vbCrLf _
        & "<div style=""width: 50em; text-align: center; margin: 0 auto 0 auto;"">" _
        & vbCrLf & vbCrLf _
        & "<div style=""width: 50em; text-align: left;"">" _
        & vbCrLf & vbCrLf

' Create the commands index for the HTML page
strPreviousLetter = "@"
strAlphabet       = "<table class=""Alphabet"" style=""width: 80%; text-align: middle; margin: 0 auto 0 auto; border: 2px solid blue;"">" & vbCrLf & "<tr>" & vbCrLf
strHTML           = "<table>" & vbCrLf
For Each objKey In dicHelpShort.Keys( )
	strCommand     = objKey
	strFirstLetter = UCase( Left( strCommand, 1 ) )
	If strFirstLetter = strPreviousLetter Then
		strHTML = strHTML & "<tr>" & vbCrLf
		strHTML = strHTML & vbTab & "<th>&nbsp;</th>" & vbCrLf
	Else
		If strFirstLetter <> "A" Then
			strHTML = strHTML & "<tr><td colspan=""3"">&nbsp;</td></tr>" & vbCrLf
			If Asc( strFirstLetter ) - Asc( strPreviousLetter ) > 1 Then
				For i = Asc( strFirstLetter ) - Asc( strPreviousLetter ) - 1 To 1 Step -1
					strAlphabet = strAlphabet & vbTab & "<th style=""color: gray;"">" & Chr( Asc( strFirstLetter ) - i ) & "</th>" & vbCrLf
				Next
			End If
		End If
		strHTML = strHTML & "<tr id=""" & strFirstLetter & """>" & vbCrLf
		strHTML = strHTML & vbTab & "<th>" & strFirstLetter & "&nbsp;&nbsp;</th>" & vbCrLf
		strAlphabet       = strAlphabet & vbTab & "<th><a href=""#" & strFirstLetter & """>" & strFirstLetter & "</a></th>" & vbCrLf
		strPreviousLetter = strFirstLetter
	End If
	If dicHelpLong.Exists( strCommand ) Then
		If dicHelpLong.Item( strCommand ) = "" Then
			strHTML = strHTML & vbTab & "<td class=""Command"">" & strCommand & "</td>" & vbCrLf
		Else
			strHTML = strHTML & vbTab & "<td class=""Command""><a href=""#" & strCommand & """>" & strCommand & "</a></td>" & vbCrLf
		End If
	End If
	strHTML = strHTML & vbTab & "<td>" & Escape( dicHelpShort.Item( strCommand ) ) & "</td>" & vbCrLf
	strHTML = strHTML & "</tr>" & vbCrLf
Next
If strFirstLetter <> "Z" Then
	For i = 1 To Asc( "Z" ) - Asc( strFirstLetter )
		strAlphabet = strAlphabet & vbTab & "<th style=""color: gray;"">" & Chr( Asc( strFirstLetter ) + i ) & "</th>" & vbCrLf
	Next
End If
strHTML     = strHTML     & "</table>" & vbCrLf & vbCrLf
strAlphabet = strAlphabet & "</tr>" & vbCrLf & "</table>" & vbCrLf & vbCrLf
strHTML     = strHead & strAlphabet & "<p>&nbsp;</p>" & vbCrLf & vbCrLf & strHTML

' Add detailed help for each command, if available
For Each objKey In dicHelpShort.Keys( )
	strCommand = objKey
	If dicHelpLong.Exists( strCommand ) Then
		strHelpLong = Escape( dicHelpLong.Item( strCommand ) )
		If strHelpLong <> "" Then
			strHTML     = strHTML & "<p>&nbsp;</p>" & vbCrLf & vbCrLf _
			            & "<h2 id=""" & strCommand & """>" & strCommand & "</h2>" & vbCrLf & vbCrLf _
			            & "<pre>" & strHelpLong & "</pre>" & vbCrLf & vbCrLf _
			            & "<div style=""text-align: center;""><a href=""#"">Back to the top of this page</a></div>" & vbCrLf & vbCrLf
		End If
	End If
Next

' Write the "footer"
strHTML = strHTML _
        & "<p>&nbsp;</p>" _
        & vbCrLf & vbCrLf _
        & "<div style=""text-align: center;"">" _
        & vbCrLf _
        & "<p>This HTML help file was generated by <a href=""http://www.robvanderwoude.com/wshexamples.php?fc=A#AllHelp"">AllHelp.vbs</a>, Version " & strScriptVer & "<br>" _
        & vbCrLf _
        & "Written by Rob van der Woude<br>" _
        & vbCrLf _
        & "<a href=""http://www.robvanderwoude.com"">http://www.robvanderwoude.com</a></p>" _
        & vbCrLf _
        & "</div>" _
        & vbCrLf & vbCrLf _
        & "</div>" _
        & vbCrLf & vbCrLf _
        & "</div>" _
        & vbCrLf & vbCrLf _
        & "</body>" _
        & vbCrLf _
        & "</html>"

'Trim leading and trailing blank lines inside PRE blocks (workaround for CR/CR/LF bug)
strHTML = TrimCrLf( strHTML )

' Write the complete HTML code to a file
Set objHTMLFile = objFSO.CreateTextFile( strFile, blnOverwrite, False )
objHTMLFile.Write strHTML
objHTMLFile.Close
Set objHTMLFile = Nothing
Set objFSO      = Nothing

'Open the HTML file
wshShell.Run """" & strFile & """", 0, False

' Close the last remaining object
Set wshShell = Nothing

' Display this script's help screen when in debugging mode
If blnDebug Then Syntax



Sub AddCommand( myCommand, myPattern1, myPattern2 )
	Dim objMatches, objMatches2, objRE, strCommand, strDebug
	strCommand = Trim( UCase( myCommand ) )
	If Not dicHelpShort.Exists( strCommand ) Then
		blnAdditional = True
		If blnDebug Then
			strDebug = "DEBUG: Trying to get help for the " & strcommand & " command:"
			WScript.Echo String( Len( strDebug ), "=" ) & vbCrLf & strDebug
		End If
		dicHelpLong.Add strCommand, GetAddedCmdHelp( strCommand )
		' Unfortunately, the regex patterns used here are for Windows 7, no guarantee they will work in other Windows versions
		Set objRE = New RegExp
		objRE.Global     = False
		objRE.IgnoreCase = False
		objRE.Pattern    = myPattern1
		Set objMatches = objRE.Execute( dicHelpLong.Item( strCommand ) )
		If blnDebug Then
			WScript.Echo "DEBUG: " & Right( Space( 3 ) & objMatches.Count, 3 ) & " match(es) for the regular expression"
			If objMatches.Count = 0 Then
				If dicHelpLong.Item( strCommand ) = "" Then
					WScript.Echo "DEBUG: Empty help text (unknown command?)"
				Else
					WScript.Echo "DEBUG: The following help text did not match:"
					WScript.Echo dicHelpLong.Item( strCommand )
				End If
			End If
			WScript.Echo String( Len( strDebug ), "=" ) & vbCrLf
		End If
		If objMatches.Count > 0 Then
			objRE.Pattern = mypattern2
			Set objMatches2 = objRE.Execute( objMatches.Item(0) )
			If objMatches2.Count > 0 Then
				dicHelpShort.Add strCommand, SuperTrim( objMatches2.Item(0) )
			End If
			Set objMatches2 = Nothing
		End If
		Set objMatches = Nothing
		Set objRE      = Nothing
	End If
End Sub



Sub AddCommandWithoutSummary( myCommand )
	Dim strCmdHelp, strCommand, strDebug, strFileDescription
	strCommand = Trim( UCase( myCommand ) )
	If Not dicHelpShort.Exists( strCommand ) Then
		blnAdditional = True
		If blnDebug Then
			strDebug = "DEBUG: Adding help for the " & strCommand & " command"
			WScript.Echo String( Len( strDebug ), "=" ) & vbCrLf & strDebug
			WScript.Echo String( Len( strDebug ), "=" ) & vbCrLf
		End If
		strCmdHelp         = GetAddedCmdHelp( strCommand )
		strFileDescription = GetFileDescription( strCommand )
		If ( strCmdHelp <> "" Or strFileDescription <> "" ) Then
			dicHelpLong.Add  strCommand, strCmdHelp
			dicHelpShort.Add strCommand, strFileDescription
		Else
			WScript.Echo DebugDisplay( "No help nor description available for " & strCommand )
		End If
	End If
End Sub



Function DebugDisplay( myString )
	Dim strMsg
	strMsg = "DEBUG: " & myString
	DebugDisplay = String( Len( strMsg ), "=" ) & vbCrLf & strMsg & vbCrLf & String( Len( strMsg ), "=" ) & vbCrLf
End Function



Function DebugDisplayHelpFound( )
	Dim strMsg
	strMsg = "====================================" _
	       & vbCrLf _
	       & "DEBUG: The following help was found:" _
	       & vbCrLf _
	       & "====================================" _
	       & vbCrLf
	For Each objKey In dicHelpShort.Keys( )
		strMsg = strMsg & Left( objKey & Space( 16 ), 16 ) & Escape( dicHelpShort.Item( objKey ) ) & vbCrLf
	Next
	strMsg = strMsg _
	       & "====================================" _
	       & vbCrLf
	DebugDisplayHelpFound = strMsg
End Function



Function Escape( myText )
	Dim strText
	strText = Replace( myText,  "&", "&amp;"  )
	strText = Replace( strText, "^", "&circ;" )
	strText = Replace( strText, "<", "&lt;"   )
	strText = Replace( strText, ">", "&gt;"   )
	strText = Replace( strText, Chr(160), "&aacute;" )
	strText = Replace( strText, Chr(181), "&Aacute;" )
	strText = Replace( strText, Chr(131), "&acirc;"  )
	strText = Replace( strText, Chr(182), "&Acirc;"  )
	strText = Replace( strText, Chr(133), "&agrave;" )
	strText = Replace( strText, Chr(183), "&Agrave;" )
	strText = Replace( strText, Chr(134), "&aring;"  )
	strText = Replace( strText, Chr(143), "&Aring;"  )
	strText = Replace( strText, Chr(198), "&atilde;" )
	strText = Replace( strText, Chr(199), "&Atilde;" )
	strText = Replace( strText, Chr(132), "&auml;"   )
	strText = Replace( strText, Chr(142), "&Auml;"   )
	strText = Replace( strText, Chr(145), "&aelig;"  )
	strText = Replace( strText, Chr(146), "&AElig;"  )
	strText = Replace( strText, Chr(135), "&ccedil;" )
	strText = Replace( strText, Chr(128), "&Ccedil;" )
	strText = Replace( strText, Chr(130), "&eacute;" )
	strText = Replace( strText, Chr(144), "&Eacute;" )
	strText = Replace( strText, Chr(136), "&ecirc;"  )
	strText = Replace( strText, Chr(210), "&Ecirc;"  )
	strText = Replace( strText, Chr(138), "&egrave;" )
	strText = Replace( strText, Chr(212), "&Egrave;" )
	strText = Replace( strText, Chr(137), "&euml;"   )
	strText = Replace( strText, Chr(211), "&Euml;"   )
	strText = Replace( strText, Chr(161), "&iacute;" )
	strText = Replace( strText, Chr(214), "&Iacute;" )
	strText = Replace( strText, Chr(140), "&icirc;"  )
	strText = Replace( strText, Chr(215), "&Icirc;"  )
	strText = Replace( strText, Chr(141), "&igrave;" )
	strText = Replace( strText, Chr(222), "&Igrave;" )
	strText = Replace( strText, Chr(139), "&iuml;"   )
	strText = Replace( strText, Chr(216), "&Iuml;"   )
	strText = Replace( strText, Chr(164), "&ntilde;" )
	strText = Replace( strText, Chr(165), "&Ntilde;" )
	strText = Replace( strText, Chr(162), "&oacute;" )
	strText = Replace( strText, Chr(224), "&Oacute;" )
	strText = Replace( strText, Chr(147), "&ocirc;"  )
	strText = Replace( strText, Chr(226), "&Ocirc;"  )
	strText = Replace( strText, Chr(149), "&ograve;" )
	strText = Replace( strText, Chr(227), "&Ograve;" )
	strText = Replace( strText, Chr(228), "&otilde;" )
	strText = Replace( strText, Chr(229), "&Otilde;" )
	strText = Replace( strText, Chr(148), "&ouml;"   )
	strText = Replace( strText, Chr(153), "&Ouml;"   )
	strText = Replace( strText, Chr(163), "&uacute;" )
	strText = Replace( strText, Chr(233), "&Uacute;" )
	strText = Replace( strText, Chr(150), "&ucirc;"  )
	strText = Replace( strText, Chr(234), "&Ucirc;"  )
	strText = Replace( strText, Chr(151), "&ugrave;" )
	strText = Replace( strText, Chr(235), "&Ugrave;" )
	strText = Replace( strText, Chr(129), "&uuml;"   )
	strText = Replace( strText, Chr(154), "&Uuml;"   )
	strText = Replace( strText, Chr(236), "&yacute;" )
	strText = Replace( strText, Chr(237), "&Yacute;" )
	strText = Replace( strText, Chr(225), "&szlig;"  )
	strText = Replace( strText, Chr(230), "&micro;"  )
	strText = Replace( strText, Chr(248), "&deg;"    )
	strText = Replace( strText, Chr(173), "&iexcl;"  )
	strText = Replace( strText, Chr(191), "&iquest;" )
	strText = Replace( strText, Chr(184), "&copy;"   )
	strText = Replace( strText, Chr(169), "&reg;"    )
	strText = Replace( strText, Chr(171), "&laquo;"  )
	strText = Replace( strText, Chr(175), "&raquo;"  )
	strText = Replace( strText, Chr(241), "&plusmn;" )
	strText = Replace( strText, Chr(244), "&para;"   )
	Escape  = strText
End Function



Function GetCommandError( )
	Dim objExec, strCommandLine, strError, wshShell
	strCommandLine = "CMD.EXE /A /C (TITLE AllHelp: TESTING&MY_DUMMY_UNKNOWN_COMMAND.EXE /? 2>&1)"
	Set wshShell = CreateObject( "Wscript.Shell" )
	Set objExec  = wshShell.Exec( strCommandLine )
	strError = objExec.StdOut.ReadAll
	objExec.Terminate
	Set objExec  = Nothing
	Set wshShell = Nothing
	GetCommandError = strError
End Function



Function GetAddedCmdHelp( myCommand )
' Get the help text for the specified command
' and format it to fit in the generated HTML page
	Dim objExec, objRE, strCommand, strCommandLine, strHelp, wshShell
	strCommand = Trim( UCase( myCommand ) )
	Set wshShell = CreateObject( "Wscript.Shell" )
	' Remove ASCII null characters; trick by Austin France
	' http://austinfrance.wordpress.com/2011/01/13/howto-vbscript-remove-chr0-null-from-a-string/
	Set objRE = New RegExp
	objRE.Global     = True
	objRE.Pattern    = "[\0]"
	objRE.IgnoreCase = False
	' Build the command line to extract the help text
	strCommandLine = "CMD.EXE /A /C (TITLE AllHelp: " & strCommand & "&" & strCommand & " /? 2>&1)"
	If strCommand = "IFMEMBER" Then
		strCommandLine = "CMD.EXE /A /C (TITLE AllHelp: IFMEMBER&IFMEMBER 2>&1)"
	End If
	If strCommand = "REGISTER-CIMPROVIDER" Then
		strCommandLine = "CMD.EXE /A /C (TITLE AllHelp: REGISTER-CIMPROVIDER&REGISTER-CIMPROVIDER -help 2>&1)"
	End If
	If strCommand = "RPCPING" Then
		' Use MORE to remove ASCII null characters
		strCommandLine = "CMD.EXE /A /C (TITLE AllHelp: RPCPING&RPCPING /? 2>&1 | MORE)"
	End If
	If strCommand = "SC" Then
		strCommandLine = "CMD.EXE /A /C (TITLE AllHelp: SC - Press Enter to continue&SC /? 2>&1)"
	End If
	If strCommand = "SUBINACL" Then
		' SUBINACL's output is incomplete Unicode and poses quite a challenge to redirect and then read;
		' so we first create an Unicode text file with 1 blank line, then append SUBINACL's output,
		' and then convert Unicode to ASCII using the TYPE command.
		Dim objFile, objFSO, strFile
		strFile = wshShell.ExpandEnvironmentStrings( "%Temp%\subinacl.txt" )
		Set objFSO   = CreateObject( "Scripting.FileSystemObject" )
		Set objFile  = objFSO.CreateTextFile( strFile, True, True )
		objFile.WriteBlankLines 1
		objFile.Close
		strCommandLine = "CMD /C (SUBINACL /? >> """ & strFile & """ 2>&1)"
		Set objExec = wshShell.Exec( strCommandLine )
		objExec.Terminate
		strCommandLine = "CMD /A /C (TYPE """ & strFile & """)"
	End If
	Set objExec = wshShell.Exec( strCommandLine )
	strHelp = objRE.Replace( objExec.StdOut.ReadAll, "" )
	objExec.Terminate
	If strCommand = "BITSADMIN" Then
		strHelp = Replace( strHelp, "SetNotifyFlags/GETNOTIFYINTERFACE", "SetNotifyFlags" & vbCrLf & "/GETNOTIFYINTERFACE" )
	End If
	' Check if the command line returned an "unknown command" error
	If strHelp = Replace( strUnknownCommand, "MY_DUMMY_UNKNOWN_COMMAND.EXE", strCommand ) Then
		If blnDebug Then DebugDisplay SuperTrim( strHelp )
		strHelp = ""
	End If
	' Clean up
	Set objRE    = Nothing
	Set objExec  = Nothing
	Set wshShell = Nothing
	' Return the result
	GetAddedCmdHelp = strHelp
End Function



Function GetCmdHelp( myCommand )
' Get the help text for the specified command
' and format it to fit in the generated HTML page
	Dim objExec, objRE1, objRE2, strCommand, strCommandLine, strHelp, wshShell
	strCommand = Trim( UCase( myCommand ) )
	' Check if there is any help available at all; if not, try <command> /? instead.
	Set objRE1 = New RegExp
	objRE1.Global     = True
	objRE1.Pattern    = """" & strCommand & " /\?"""
	objRE1.IgnoreCase = True
	' Remove ASCII null characters; trick by Austin France
	' http://austinfrance.wordpress.com/2011/01/13/howto-vbscript-remove-chr0-null-from-a-string/
	Set objRE2 = New RegExp
	objRE2.Global     = True
	objRE2.Pattern    = "[\0]"
	objRE2.IgnoreCase = False
	' Build the command line to extract the help text
	strCommandLine = "CMD.EXE /A /C (TITLE AllHelp: " & strCommand & "&HELP " & strCommand & ")"
	If strCommand = "SC" Then
		strCommandLine = "CMD.EXE /A /C (TITLE AllHelp: SC - Press Enter to continue&HELP SC)"
	End If
	Set wshShell = CreateObject( "Wscript.Shell" )
	Set objExec = wshShell.Exec( strCommandLine )
	strHelp = objRE2.Replace( objExec.StdOut.ReadAll, "" )
	objExec.Terminate
	' Check if there is any help available at all; if not, try <command> /? instead.
	If objRE1.Test( strHelp ) Then strHelp = GetAddedCmdHelp( strCommand )
	' Clean up
	Set objRE1   = Nothing
	Set objRE2   = Nothing
	Set objExec  = Nothing
	Set wshShell = Nothing
	' Return the result
	GetCmdHelp = strHelp
End Function



Function GetFileDescription( myCommand )
	GetFileDescription = ""
	Dim objFolder, objFSO, objItem, objShell, strFile, wshShell
	Set objFSO = CreateObject( "Scripting.FileSystemObject" )
	Set wshShell = CreateObject( "Wscript.Shell" )
	strFile = wshShell.ExpandEnvironmentStrings( "%windir%\system32\" & myCommand & ".exe" )
	If objFSO.FileExists( strFile ) Then
		Set objShell  = CreateObject( "Shell.Application" )
		Set objFolder = objShell.NameSpace( wshShell.ExpandEnvironmentStrings( "%windir%\system32" ) )
		For Each objItem In objFolder.Items
			If UCase( objItem.Name ) = UCase( myCommand & ".exe" ) Then
				GetFileDescription = objFolder.GetDetailsOf( objItem, 34 )
			End If
		Next
		Set objFolder = Nothing
		Set objShell  = Nothing
	End If
	Set objFSO   = Nothing
	Set wshShell = Nothing
End Function



Function GetOSLanguage( myLocale )
' Get the description for the OS language/locale number
	Dim i, arrOSLanguage(20490)
	For i = 0 To 20490
		arrOSLanguage(i) = ""
	Next
	arrOSLanguage(1)     = "Arabic"
	arrOSLanguage(4)     = "Chinese (Simplified) - China"
	arrOSLanguage(9)     = "English"
	arrOSLanguage(1025)  = "Arabic - Saudi Arabia"
	arrOSLanguage(1026)  = "Bulgarian"
	arrOSLanguage(1027)  = "Catalan"
	arrOSLanguage(1028)  = "Chinese (Traditional) - Taiwan"
	arrOSLanguage(1029)  = "Czech"
	arrOSLanguage(1030)  = "Danish"
	arrOSLanguage(1031)  = "German - Germany"
	arrOSLanguage(1032)  = "Greek"
	arrOSLanguage(1033)  = "English - United States"
	arrOSLanguage(1034)  = "Spanish - Traditional Sort"
	arrOSLanguage(1035)  = "Finnish"
	arrOSLanguage(1036)  = "French - France"
	arrOSLanguage(1037)  = "Hebrew"
	arrOSLanguage(1038)  = "Hungarian"
	arrOSLanguage(1039)  = "Icelandic"
	arrOSLanguage(1040)  = "Italian - Italy"
	arrOSLanguage(1041)  = "Japanese"
	arrOSLanguage(1042)  = "Korean"
	arrOSLanguage(1043)  = "Dutch - Netherlands"
	arrOSLanguage(1044)  = "Norwegian - Bokmal"
	arrOSLanguage(1045)  = "Polish"
	arrOSLanguage(1046)  = "Portuguese - Brazil"
	arrOSLanguage(1047)  = "Rhaeto-Romanic"
	arrOSLanguage(1048)  = "Romanian"
	arrOSLanguage(1049)  = "Russian"
	arrOSLanguage(1050)  = "Croatian"
	arrOSLanguage(1051)  = "Slovak"
	arrOSLanguage(1052)  = "Albanian"
	arrOSLanguage(1053)  = "Swedish"
	arrOSLanguage(1054)  = "Thai"
	arrOSLanguage(1055)  = "Turkish"
	arrOSLanguage(1056)  = "Urdu"
	arrOSLanguage(1057)  = "Indonesian"
	arrOSLanguage(1058)  = "Ukrainian"
	arrOSLanguage(1059)  = "Belarusian"
	arrOSLanguage(1060)  = "Slovenian"
	arrOSLanguage(1061)  = "Estonian"
	arrOSLanguage(1062)  = "Latvian"
	arrOSLanguage(1063)  = "Lithuanian"
	arrOSLanguage(1065)  = "Persian"
	arrOSLanguage(1066)  = "Vietnamese"
	arrOSLanguage(1069)  = "Basque"
	arrOSLanguage(1070)  = "Serbian"
	arrOSLanguage(1071)  = "Macedonian (FYROM)"
	arrOSLanguage(1072)  = "Sutu"
	arrOSLanguage(1073)  = "Tsonga"
	arrOSLanguage(1074)  = "Tswana"
	arrOSLanguage(1076)  = "Xhosa"
	arrOSLanguage(1077)  = "Zulu"
	arrOSLanguage(1078)  = "Afrikaans"
	arrOSLanguage(1080)  = "Faeroese"
	arrOSLanguage(1081)  = "Hindi"
	arrOSLanguage(1082)  = "Maltese"
	arrOSLanguage(1084)  = "Gaelic"
	arrOSLanguage(1085)  = "Yiddish"
	arrOSLanguage(1086)  = "Malay - Malaysia"
	arrOSLanguage(2049)  = "Arabic - Iraq"
	arrOSLanguage(2052)  = "Chinese (Simplified) - PRC"
	arrOSLanguage(2055)  = "German - Switzerland"
	arrOSLanguage(2057)  = "English - United Kingdom"
	arrOSLanguage(2058)  = "Spanish - Mexico"
	arrOSLanguage(2060)  = "French - Belgium"
	arrOSLanguage(2064)  = "Italian - Switzerland"
	arrOSLanguage(2067)  = "Dutch - Belgium"
	arrOSLanguage(2068)  = "Norwegian - Nynorsk"
	arrOSLanguage(2070)  = "Portuguese - Portugal"
	arrOSLanguage(2072)  = "Romanian - Moldova"
	arrOSLanguage(2073)  = "Russian - Moldova"
	arrOSLanguage(2074)  = "Serbian - Latin"
	arrOSLanguage(2077)  = "Swedish - Finland"
	arrOSLanguage(3073)  = "Arabic - Egypt"
	arrOSLanguage(3076)  = "Chinese (Traditional) - Hong Kong SAR"
	arrOSLanguage(3079)  = "German - Austria"
	arrOSLanguage(3081)  = "English - Australia"
	arrOSLanguage(3082)  = "Spanish - International Sort"
	arrOSLanguage(3084)  = "French - Canada"
	arrOSLanguage(3098)  = "Serbian - Cyrillic"
	arrOSLanguage(4097)  = "Arabic - Libya"
	arrOSLanguage(4100)  = "Chinese (Simplified) - Singapore"
	arrOSLanguage(4103)  = "German - Luxembourg"
	arrOSLanguage(4105)  = "English - Canada"
	arrOSLanguage(4106)  = "Spanish - Guatemala"
	arrOSLanguage(4108)  = "French - Switzerland"
	arrOSLanguage(5121)  = "Arabic - Algeria"
	arrOSLanguage(5127)  = "German - Liechtenstein"
	arrOSLanguage(5129)  = "English - New Zealand"
	arrOSLanguage(5130)  = "Spanish - Costa Rica"
	arrOSLanguage(5132)  = "French - Luxembourg"
	arrOSLanguage(6145)  = "Arabic - Morocco"
	arrOSLanguage(6153)  = "English - Ireland"
	arrOSLanguage(6154)  = "Spanish - Panama"
	arrOSLanguage(7169)  = "Arabic - Tunisia"
	arrOSLanguage(7177)  = "English - South Africa"
	arrOSLanguage(7178)  = "Spanish - Dominican Republic"
	arrOSLanguage(8193)  = "Arabic - Oman"
	arrOSLanguage(8201)  = "English - Jamaica"
	arrOSLanguage(8202)  = "Spanish - Venezuela"
	arrOSLanguage(9217)  = "Arabic - Yemen"
	arrOSLanguage(9226)  = "Spanish - Colombia"
	arrOSLanguage(10241) = "Arabic - Syria"
	arrOSLanguage(10249) = "English - Belize"
	arrOSLanguage(10250) = "Spanish - Peru"
	arrOSLanguage(11265) = "Arabic - Jordan"
	arrOSLanguage(11273) = "English - Trinidad"
	arrOSLanguage(11274) = "Spanish - Argentina"
	arrOSLanguage(12289) = "Arabic - Lebanon"
	arrOSLanguage(12298) = "Spanish - Ecuador"
	arrOSLanguage(13313) = "Arabic - Kuwait"
	arrOSLanguage(13322) = "Spanish - Chile"
	arrOSLanguage(14337) = "Arabic - U.A.E."
	arrOSLanguage(14346) = "Spanish - Uruguay"
	arrOSLanguage(15361) = "Arabic - Bahrain"
	arrOSLanguage(15370) = "Spanish - Paraguay"
	arrOSLanguage(16385) = "Arabic - Qatar"
	arrOSLanguage(16394) = "Spanish - Bolivia"
	arrOSLanguage(17418) = "Spanish - El Salvador"
	arrOSLanguage(18442) = "Spanish - Honduras"
	arrOSLanguage(19466) = "Spanish - Nicaragua"
	arrOSLanguage(20490) = "Spanish - Puerto Rico"
	If Trim( myLocale ) = "" Then
		GetOSLanguage = arrOSLanguage( GetLocale( ) )
	Else
		GetOSLanguage = arrOSLanguage( myLocale )
	End If
End Function



Function SortDictionary( objDict, intSort )
' Sort a Dictionary object
' Code found on http://support.microsoft.com/kb/246067
	Dim arrDict( )
	Dim objKey
	Dim strKey, strItem
	Dim i,j,intCount
	
	' Get the dictionary count
	intCount = objDict.Count
	
	' We need more than one item to warrant sorting
	If intCount > 1 Then
		' create an array to store dictionary information
		ReDim arrDict( intCount, 2 )
		i = 0
		' Populate the string array
		For Each objKey In objDict
			arrDict( i, dictKey )  = CStr( objKey )
			arrDict( i, dictItem ) = CStr( objDict( objKey ) )
			i = i + 1
		Next

		' Perform a shell sort of the string array
		For i = 0 to ( intCount - 2 )
			For j = i to ( intCount - 1 )
				If StrComp( arrDict( i, intSort ), arrDict( j, intSort ),vbTextCompare ) > 0 Then
					strKey  = arrDict( i, dictKey )
					strItem = arrDict( i, dictItem )
					arrDict( i, dictKey )  = arrDict( j, dictKey )
					arrDict( i, dictItem ) = arrDict( j, dictItem )
					arrDict( j, dictKey )  = strKey
					arrDict( j, dictItem ) = strItem
				End If
			Next
		Next

		' Erase the contents of the dictionary object
		objDict.RemoveAll

		' Repopulate the dictionary with the sorted information
		For i = 0 to ( intCount - 1 )
			objDict.Add arrDict( i, dictKey ), arrDict( i, dictItem )
		Next
	End If
End Function



Function SuperTrim( myText )
	Dim strText
	strText = Replace( myText,  vbCr, " " )
	strText = Replace( strText, vbLf, " " )
	While ( InStr( strText, "  " ) )
		strText = Replace( strText, "  ", " " )
	Wend
	SuperTrim = Trim( strText )
End Function



Sub Syntax
' Display help and, optionally, debugging information
	strMsg = strMsg & vbCrLf _
	       & "AllHelp.vbs,  Version " & strScriptVer _
	       & vbCrLf _
	       & "Display help for ""all"" Windows commands, for" _
	       & vbCrLf _
	       & "the current Windows installion and language." _
	       & vbCrLf & vbCrLf _
	       & "Usage:  ALLHELP.VBS  [ ""outputfile"" ]  [ /Y ]  [ /WHO ]  [ /DEBUG ]" _
	       & vbCrLf & vbCrLf _
	       & "Where:  ""outputfile""   is the fully qualified path of the HTML file to be" _
	       & vbCrLf _
	       & "                       created (default: ""allhelp.html"" in ""My Documents"")" _
	       & vbCrLf _
	       & "        /DEBUG         display DEBUGging information" _
	       & vbCrLf _
	       & "        /WHO           show commands listed in Windows' Help Only" _
	       & vbCrLf _
	       & "        /Y             overwrite existing file (default: don't overwrite)" _
	       & vbCrLf & vbCrLf _
	       & "Notes:  DISKPART and OPENFILES are protected by UAC; in order to retrieve" _
	       & vbCrLf _
	       & "        their help texts, run this script with elevated privileges." _
	       & vbCrLf _
	       & "        SC help requires keyboard input; its window title will ask you to" _
	       & vbCrLf _
	       & "        press Enter." _
	       & vbCrLf & vbCrLf _
	       & "Written by Rob van der Woude" _
	       & vbCrLf _
	       & "http://www.robvanderwoude.com"
	WScript.Echo strMsg
	WScript.Quit 1
End Sub



Function TrimCrLf( myString )
	Dim objMatches, objRE, strResult
	strResult = Trim( myString )
	If strResult = "" Then Exit Function
	Set objRE = New RegExp
	objRE.Global  = True
	objRE.Pattern = "<pre>(" & vbCr & "|" & vbLf & ")+"
	strResult     = objRE.Replace( strResult, "<pre>" )
	objRE.Pattern = "(" & vbCr & "|" & vbLf & ")+</pre>"
	strResult     = objRE.Replace( strResult, "</pre>" )
	Set objRE = Nothing
	strResult = Replace( strResult, vbCr & vbCrLf, vbCrLf )
	TrimCrLf = strResult
End Function
